@@ -41,11 +41,15 @@ struct _ViewAutoDrawerPrivate
 
    gboolean opened;
 
+   gboolean fill;
+   gint offset;
+
    guint delayConnection;
    guint delayValue;
    guint overlapPixels;
    guint noOverlapPixels;
 
+   GtkWidget *over;
    GtkWidget *evBox;
 };
 
@@ -84,9 +88,12 @@ ViewAutoDrawerEnforce(ViewAutoDrawer *that, // IN
       return;
    }
 
+   g_assert(priv->over != NULL);
+   g_assert(GTK_IS_WIDGET(priv->over));
+
    ViewOvBox_SetMin(VIEW_OV_BOX(that), priv->noOverlapPixels);
    fraction = priv->opened
-      ? 1 : (double)priv->overlapPixels / GTK_WIDGET(that)->allocation.height;
+      ? 1 : ((double)priv->overlapPixels / priv->over->allocation.height);
    if (!animate) {
       ViewOvBox_SetFraction(VIEW_OV_BOX(that), fraction);
    }
@@ -213,8 +220,6 @@ ViewAutoDrawerUpdate(ViewAutoDrawer *that, // IN
       g_assert(grabbed);
 
       if (GTK_IS_MENU(grabbed)) {
-         GtkWidget *menuItemParent;
-
          /*
           * With cascading menus, the deepest menu owns the grab. Traverse the
           * menu hierarchy up until we reach the attach widget for the whole
@@ -222,8 +227,19 @@ ViewAutoDrawerUpdate(ViewAutoDrawer *that, // IN
           */
 
          for (;;) {
-            grabbed = gtk_menu_get_attach_widget(GTK_MENU(grabbed));
-            g_return_if_fail(grabbed);
+            GtkWidget *menuAttach;
+            GtkWidget *menuItemParent;
+
+            menuAttach = gtk_menu_get_attach_widget(GTK_MENU(grabbed));
+            if (!menuAttach) {
+               /*
+                * It is unfortunately not mandatory for a menu to have a proper
+                * attach widget set.
+                */
+               break;
+            }
+
+            grabbed = menuAttach;
             if (!GTK_IS_MENU_ITEM(grabbed)) {
                break;
             }
@@ -433,6 +449,41 @@ ViewAutoDrawerSetOver(ViewOvBox *ovBox,  // IN
    if (oldChild) {
       g_object_unref(oldChild);
    }
+
+   priv->over = widget;
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * ViewAutoDrawerRefreshPacking --
+ *
+ *      Sets the actual packing values for fill, expand, and packing
+ *      given internal settings.
+ *
+ * Results:
+ *      None
+ *
+ * Side effects:
+ *      Drawer state is updated.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+ViewAutoDrawerRefreshPacking(ViewAutoDrawer *that) // IN
+{
+   gboolean expand;
+   gboolean fill;
+   guint padding;
+
+   expand = (that->priv->fill || (that->priv->offset < 0));
+   fill = that->priv->fill;
+   padding = (expand || fill) ? 0 : that->priv->offset;
+   
+   gtk_box_set_child_packing(GTK_BOX(that), that->priv->evBox,
+                             expand, fill, padding, GTK_PACK_START);
 }
 
 
@@ -471,6 +522,9 @@ ViewAutoDrawerInit(GTypeInstance *instance, // IN
    priv->overlapPixels = 0;
    priv->noOverlapPixels = 1;
 
+   priv->fill = TRUE;
+   priv->offset = -1;
+
    priv->evBox = gtk_event_box_new();
    gtk_widget_show(priv->evBox);
    VIEW_OV_BOX_CLASS(parentClass)->set_over(VIEW_OV_BOX(that), priv->evBox);
@@ -487,6 +541,8 @@ ViewAutoDrawerInit(GTypeInstance *instance, // IN
 
    /* This change happens programmatically. Always react to it immediately. */
    ViewAutoDrawerUpdate(that, TRUE);
+
+   ViewAutoDrawerRefreshPacking(that);
 }
 
 
@@ -782,6 +838,7 @@ ViewAutoDrawer_SetPinned(ViewAutoDrawer *that, // IN
  *
  *      Set whether the Over widget of the AutoDrawer should fill the full
  *      width of the AutoDrawer or just occupy the minimum space it needs.
+ *      A value of TRUE overrides offset settings.
  *
  * Results:
  *      None
@@ -798,6 +855,35 @@ ViewAutoDrawer_SetFill(ViewAutoDrawer *that, // IN
 {
    g_return_if_fail(VIEW_IS_AUTODRAWER(that));
 
-   gtk_box_set_child_packing(GTK_BOX(that), that->priv->evBox,
-                             TRUE, fill, 0, GTK_PACK_START);
+   that->priv->fill = fill;
+   ViewAutoDrawerRefreshPacking(that);
+}
+
+
+/*
+ *-----------------------------------------------------------------------------
+ *
+ * ViewAutoDrawer_SetOffset --
+ *
+ *      Set the drawer's X offset, or distance in pixels from the left side.
+ *      If offset is -1, the drawer will be centered.  If fill has been set
+ *      TRUE by SetFill, these settings will have no effect.
+ *
+ * Results:
+ *      None
+ *
+ * Side effects:
+ *      Drawer state is updated.
+ *
+ *-----------------------------------------------------------------------------
+ */
+
+void
+ViewAutoDrawer_SetOffset(ViewAutoDrawer *that, // IN
+                         gint offset)          // IN
+{
+   g_return_if_fail(VIEW_IS_AUTODRAWER(that));
+
+   that->priv->offset = offset;
+   ViewAutoDrawerRefreshPacking(that);
 }
